

from RouToolPa.Tools.LinuxTools.CGAS import CGAS
from RouToolPa.Tools.LinuxTools.Download import *
from RouToolPa.Tools.LinuxTools.Valgrind import Valgrind


Axel = Axel()
Wget = Wget()
CGAS = CGAS()
Valgrind = Valgrind()
