#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Scaffolders.LRNAScaffolder import LRNAScaffolder
from RouToolPa.Tools.Scaffolders.SSPACE import SSPACE


SSPACE = SSPACE()
LRNAScaffolder = LRNAScaffolder()
