__author__ = 'mahajrod'

from RouToolPa.Routines.GO import GORoutines
from RouToolPa.Routines.VCF import VCFRoutines
from RouToolPa.Routines.Math import MathRoutines
from RouToolPa.Routines.File import FileRoutines
from RouToolPa.Routines.Tree import TreeRoutines
from RouToolPa.Routines.NCBI import NCBIRoutines
from RouToolPa.Routines.Math import SmoothRoutines
from RouToolPa.Routines.MtDNA import MtDNARoutines
from RouToolPa.Routines.Fastq import FastQRoutines
from RouToolPa.Routines.Primer import PrimerRoutines
from RouToolPa.Routines.EggNOG import EggNOGRoutines
from RouToolPa.Routines.Project import ProjectRoutines
from RouToolPa.Routines.Ensembl import EnsemblRoutines
from RouToolPa.Routines.TreeFam import TreeFamRoutines
from RouToolPa.Routines.Drawing import DrawingRoutines
from RouToolPa.Routines.Sequence import SequenceRoutines
from RouToolPa.Routines.Evolution import EvolutionRoutines
from RouToolPa.Routines.Alignment import AlignmentRoutines
from RouToolPa.Routines.Expression import ExpressionRoutines
from RouToolPa.Routines.Matplotlib import MatplotlibRoutines
from RouToolPa.Routines.Annotations import AnnotationsRoutines
from RouToolPa.Routines.Phylogenetics import PhylogeneticsRoutines
from RouToolPa.Routines.SequenceCluster import SequenceClusterRoutines
from RouToolPa.Routines.MultipleAlignment import MultipleAlignmentRoutines


GORoutines = GORoutines()
VCFRoutines = VCFRoutines()
MathRoutines = MathRoutines()
FileRoutines = FileRoutines()
TreeRoutines = TreeRoutines()
NCBIRoutines = NCBIRoutines()
MtDNARoutines = MtDNARoutines()
FastQRoutines = FastQRoutines()
SmoothRoutines = SmoothRoutines()
PrimerRoutines = PrimerRoutines()
EggNOGRoutines = EggNOGRoutines()
ProjectRoutines = ProjectRoutines()
EnsemblRoutines = EnsemblRoutines()
TreeFamRoutines = TreeFamRoutines()
DrawingRoutines = DrawingRoutines()
SequenceRoutines = SequenceRoutines()
EvolutionRoutines = EvolutionRoutines()
AlignmentRoutines = AlignmentRoutines()
ExpressionRoutines = ExpressionRoutines()
MatplotlibRoutines = MatplotlibRoutines()
AnnotationsRoutines = AnnotationsRoutines()
PhylogeneticsRoutines = PhylogeneticsRoutines()
SequenceClusterRoutines = SequenceClusterRoutines()
MultipleAlignmentRoutines = MultipleAlignmentRoutines()
