#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.GapClosers.Sealer import Sealer
from RouToolPa.Tools.GapClosers.GapFiller import GapFiller

Sealer = Sealer()
GapFiller = GapFiller()
