__author__ = 'mahajrod'

from RouToolPa.Tools.GATK4.SortVcf4 import SortVcf4
from RouToolPa.Tools.GATK4.CombineGVCFs4 import CombineGVCFs4
from RouToolPa.Tools.GATK4.GenotypeGVCFs4 import GenotypeGVCFs4
from RouToolPa.Tools.GATK4.SelectVariants4 import SelectVariants4
#from RouToolPa.Tools.GATK4.CombineVariants4 import CombineVariants4
from RouToolPa.Tools.GATK4.HaplotypeCaller4 import HaplotypeCaller4
from RouToolPa.Tools.GATK4.GenomicsDBImport4 import GenomicsDBImport4
from RouToolPa.Tools.GATK4.ValidateVariants4 import ValidateVariants4
from RouToolPa.Tools.GATK4.BaseRecalibrator4 import BaseRecalibrator4
#from RouToolPa.Tools.GATK4.UnifiedGenotyper4 import UnifiedGenotyper4
#from RouToolPa.Tools.GATK4.AnalyzeCovariates4 import AnalyzeCovariates4
from RouToolPa.Tools.GATK4.VariantFiltration4 import VariantFiltration4
#from RouToolPa.Tools.GATK4.FastaAlternateReferenceMaker4 import FastaAlternateReferenceMaker4

SortVcf4 = SortVcf4()
CombineGVCFs4 = CombineGVCFs4()
GenotypeGVCFs4 = GenotypeGVCFs4()
SelectVariants4 = SelectVariants4()
#CombineVariants4 = CombineVariants4()
HaplotypeCaller4 = HaplotypeCaller4()
GenomicsDBImport4 = GenomicsDBImport4()
ValidateVariants4 = ValidateVariants4()
#BaseRecalibrator4 = BaseRecalibrator4()
#UnifiedGenotyper4 = UnifiedGenotyper4()
#AnalyzeCovariates4 = AnalyzeCovariates4()
VariantFiltration4 = VariantFiltration4()
#FastaAlternateReferenceMaker4 = FastaAlternateReferenceMaker4()
