#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Assemblers.BionanoSolve import BionanoSolve
from RouToolPa.Tools.Assemblers.Quast import Quast
from RouToolPa.Tools.Assemblers.Spades import Spades
from RouToolPa.Tools.Assemblers.MaSuRCA import MaSuRCA
from RouToolPa.Tools.Assemblers.Supernova import Supernova


Quast = Quast()
Spades = Spades()
MaSuRCA = MaSuRCA()
Supernova = Supernova()
BionanoSolve = BionanoSolve()
