__author__ = 'mahajrod'

"""
Format of adapters_PE:
{"LibraryType": ["common_part", "A1_adapter", "A2_adapter"]}
"""

adapters_PE = {"ScriptSeq_v2": ["AGATCGGAAGAG",
                                "AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC",
                                "AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGTA"],
               "TrueSeq": ["AGATCGGAAGAGC"]
               }
