__author__ = 'mahajrod'

from RouToolPa.Tools.MultipleAlignment.GUIDANCE2 import GUIDANCE2
from RouToolPa.Tools.MultipleAlignment.PRANK import PRANK
from RouToolPa.Tools.MultipleAlignment.MAFFT import MAFFT
from RouToolPa.Tools.MultipleAlignment.SWAMP import SWAMP


PRANK = PRANK()
MAFFT = MAFFT()
SWAMP = SWAMP()
GUIDANCE2 = GUIDANCE2()
