#!/usr/bin/env python

from collections import OrderedDict

LAST_TAB_COLS = OrderedDict({
                             "score": 0,
                             "target_id":   1,
                             "target_start": 2,
                             "target_hit_len": 3,
                             "target_strand": 4,
                             "target_len": 5,
                             "query_id": 6,
                             "query_start": 7,
                             "query_hit_len": 8,
                             "query_strand": 9,
                             "query_len": 10,
                             "alignment": 11,
                             "EG2": 12,
                             "E": 13,
                              })
